"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "HorizontalBarChart", {
    enumerable: true,
    get: function() {
        return HorizontalBarChart;
    }
});
const _interop_require_wildcard = require("@swc/helpers/_/_interop_require_wildcard");
const _react = /*#__PURE__*/ _interop_require_wildcard._(require("react"));
const _useHorizontalBarChartStylesstyles = require("./useHorizontalBarChartStyles.styles");
const _index = require("./index");
const _localeutil = require("../../utilities/locale-util");
const _index1 = require("../../utilities/index");
const _reactutilities = require("@fluentui/react-utilities");
const _reacttheme = require("@fluentui/react-theme");
const _reacttabster = require("@fluentui/react-tabster");
const _ChartPopover = require("../CommonComponents/ChartPopover");
const _FocusableTooltipText = require("../../utilities/FocusableTooltipText");
const HorizontalBarChart = /*#__PURE__*/ _react.forwardRef((props, forwardedRef)=>{
    let _barHeight;
    //let _classNames: IProcessedStyleSet<IHorizontalBarChartStyles>;
    const _uniqLineText = '_HorizontalLine_' + Math.random().toString(36).substring(7);
    const _refArray = [];
    let _calloutAnchorPoint;
    const _isRTL = (0, _index1.useRtl)();
    const barChartSvgRef = /*#__PURE__*/ _react.createRef();
    const _emptyChartId = (0, _reactutilities.useId)('_HBC_empty');
    const [hoverValue, setHoverValue] = _react.useState('');
    const [lineColor, setLineColor] = _react.useState('');
    const [legend, setLegend] = _react.useState('');
    const [xCalloutValue, setXCalloutValue] = _react.useState('');
    const [yCalloutValue, setYCalloutValue] = _react.useState('');
    const [barCalloutProps, setBarCalloutProps] = _react.useState();
    const [barSpacingInPercent, setBarSpacingInPercent] = _react.useState(0);
    const [isPopoverOpen, setPopoverOpen] = _react.useState(false);
    const [clickPosition, setClickPosition] = _react.useState({
        x: 0,
        y: 0
    });
    function _refCallback(element, legendTitle) {
        _refArray.push({
            index: legendTitle,
            refElement: element
        });
    }
    function _hoverOn(event, hoverVal, point) {
        if ((!isPopoverOpen || legend !== point.legend) && _calloutAnchorPoint !== point) {
            _calloutAnchorPoint = point;
            updatePosition(event.clientX, event.clientY);
            setHoverValue(hoverVal);
            setLineColor(point.color);
            setLegend(point.legend);
            setXCalloutValue(point.xAxisCalloutData);
            setYCalloutValue(point.yAxisCalloutData);
            setBarCalloutProps(point);
        // ToDo - Confirm setting multiple state variables like this is performant.
        }
    }
    function _hoverOff() {
    /*ToDo. To fix*/ }
    const _handleChartMouseLeave = ()=>{
        _calloutAnchorPoint = null;
        if (isPopoverOpen) {
            setPopoverOpen(false);
            setHoverValue('');
            setLineColor('');
            setLegend('');
        }
    };
    const _adjustProps = ()=>{
        _barHeight = props.barHeight || 12;
    };
    const _getChartDataText = (data)=>{
        /* return props.barChartCustomData ? (
        <div role="text">{props.barChartCustomData(data)}</div>
      ) : ( */ return _getDefaultTextData(data);
    //)
    };
    function _getDefaultTextData(data) {
        const { culture } = props;
        const chartDataMode = props.chartDataMode || 'default';
        const chartData = data.chartData[0];
        const x = chartData.horizontalBarChartdata.x;
        const y = chartData.horizontalBarChartdata.y;
        const accessibilityData = (0, _index1.getAccessibleDataObject)(data.chartDataAccessibilityData, 'text', false);
        switch(chartDataMode){
            case 'default':
                return /*#__PURE__*/ _react.createElement("div", {
                    className: classes.chartTitleRight,
                    ...accessibilityData
                }, (0, _localeutil.convertToLocaleString)(x, culture));
            case 'fraction':
                return /*#__PURE__*/ _react.createElement("div", accessibilityData, /*#__PURE__*/ _react.createElement("span", {
                    className: classes.chartTitleRight
                }, (0, _localeutil.convertToLocaleString)(x, culture)), /*#__PURE__*/ _react.createElement("span", {
                    className: classes.chartDataTextDenominator
                }, ' / ' + (0, _localeutil.convertToLocaleString)(y, culture)));
            case 'percentage':
                const dataRatioPercentage = `${(0, _localeutil.convertToLocaleString)(Math.round(x / y * 100), culture)}%`;
                return /*#__PURE__*/ _react.createElement("div", {
                    className: classes.chartTitleRight,
                    ...accessibilityData
                }, dataRatioPercentage);
        }
    }
    function _createBenchmark(data) {
        const totalData = data.chartData[0].horizontalBarChartdata.y;
        const benchmarkData = data.chartData[0].data;
        const benchmarkRatio = Math.round((benchmarkData ? benchmarkData : 0) / totalData * 100);
        const benchmarkStyles = {
            left: 'calc(' + benchmarkRatio + '% - 4px)'
        };
        return /*#__PURE__*/ _react.createElement("div", {
            className: classes.benchmarkContainer
        }, /*#__PURE__*/ _react.createElement("div", {
            className: classes.triangle,
            style: benchmarkStyles
        }));
    }
    /**
   * This functions returns an array of <rect> elements, which form the bars
   * For each bar an x value, and a width needs to be specified
   * The computations are done based on percentages
   * Extra margin is also provided, in the x value to provide some spacing in between the bars
   */ function _createBars(data) {
        var _data_chartData;
        const noOfBars = ((_data_chartData = data.chartData) === null || _data_chartData === void 0 ? void 0 : _data_chartData.reduce((count, point)=>count += (point.data || 0) > 0 ? 1 : 0, 0)) || 1;
        const totalMarginPercent = barSpacingInPercent * (noOfBars - 1);
        const defaultColors = [
            _reacttheme.tokens.colorPaletteBlueForeground2,
            _reacttheme.tokens.colorPaletteCornflowerForeground2,
            _reacttheme.tokens.colorPaletteDarkGreenForeground2,
            _reacttheme.tokens.colorPaletteNavyForeground2,
            _reacttheme.tokens.colorPaletteDarkOrangeForeground2
        ];
        // calculating starting point of each bar and it's range
        const startingPoint = [];
        const total = data.chartData.reduce((acc, point)=>acc + (point.horizontalBarChartdata.x ? point.horizontalBarChartdata.x : 0), 0);
        let prevPosition = 0;
        let value = 0;
        let sumOfPercent = 0;
        data.chartData.map((point, index)=>{
            const pointData = point.horizontalBarChartdata.x ? point.horizontalBarChartdata.x : 0;
            value = pointData / total * 100;
            if (value < 0) {
                value = 0;
            } else if (value < 1 && value !== 0) {
                value = 1;
            }
            sumOfPercent += value;
            return sumOfPercent;
        });
        /**
     * The %age of the space occupied by the margin needs to subtracted
     * while computing the scaling ratio, since the margins are not being
     * scaled down, only the data is being scaled down from a higher percentage to lower percentage
     * Eg: 95% of the space is taken by the bars, 5% by the margins
     * Now if the sumOfPercent is 120% -> This needs to be scaled down to 95%, not 100%
     * since that's only space available to the bars
     */ const scalingRatio = sumOfPercent !== 0 ? (sumOfPercent - totalMarginPercent) / 100 : 1;
        const bars = data.chartData.map((point, index)=>{
            const color = point.color ? point.color : defaultColors[Math.floor(Math.random() * 4 + 1)];
            const pointData = point.horizontalBarChartdata.x ? point.horizontalBarChartdata.x : 0;
            if (index > 0) {
                prevPosition += value;
            }
            value = pointData / total * 100;
            if (value < 0) {
                value = 0;
            } else if (value < 1 && value !== 0) {
                value = 1 / scalingRatio;
            } else {
                value = value / scalingRatio;
            }
            startingPoint.push(prevPosition);
            const xValue = point.horizontalBarChartdata.x;
            const placeholderIndex = 1;
            // Render bar label instead of placeholder bar for absolute-scale variant
            if (index === placeholderIndex && props.variant === _index.HorizontalBarChartVariant.AbsoluteScale) {
                if (props.hideLabels) {
                    return /*#__PURE__*/ _react.createElement("text", {
                        key: index
                    });
                }
                const barValue = data.chartData[0].horizontalBarChartdata.x;
                return /*#__PURE__*/ _react.createElement("text", {
                    key: index,
                    x: `${_isRTL ? 100 - startingPoint[index] : startingPoint[index]}%`,
                    y: _barHeight / 2,
                    dominantBaseline: "central",
                    transform: `translate(${_isRTL ? -4 : 4})`,
                    className: classes.barLabel,
                    "aria-hidden": true
                }, (0, _index1.formatValueWithSIPrefix)(barValue));
            }
            return /*#__PURE__*/ _react.createElement("rect", {
                key: index,
                x: `${_isRTL ? 100 - startingPoint[index] - value - index * barSpacingInPercent : startingPoint[index] + index * barSpacingInPercent}%`,
                y: 0,
                "data-is-focusable": point.legend !== '' ? true : false,
                width: value + '%',
                height: _barHeight,
                fill: color,
                onMouseOver: point.legend !== '' ? (event)=>_hoverOn(event, xValue, point) : undefined,
                onFocus: point.legend !== '' ? (event)=>_hoverOn.bind(event, xValue, point) : undefined,
                role: "img",
                "aria-label": _getAriaLabel(point),
                onBlur: _hoverOff,
                onMouseLeave: _hoverOff,
                className: classes.barWrapper,
                tabIndex: point.legend !== '' ? 0 : undefined
            });
        });
        return bars;
    }
    const _getAriaLabel = (point)=>{
        var _point_callOutAccessibilityData;
        const legend = point.xAxisCalloutData || point.legend;
        const yValue = point.yAxisCalloutData || (point.horizontalBarChartdata ? `${point.horizontalBarChartdata.x}/${point.horizontalBarChartdata.y}` : 0);
        return ((_point_callOutAccessibilityData = point.callOutAccessibilityData) === null || _point_callOutAccessibilityData === void 0 ? void 0 : _point_callOutAccessibilityData.ariaLabel) || (legend ? `${legend}, ` : '') + `${yValue}.`;
    };
    function _isChartEmpty() {
        return !(props.data && props.data.length > 0);
    }
    function updatePosition(newX, newY) {
        const threshold = 1; // Set a threshold for movement
        const { x, y } = clickPosition;
        // Calculate the distance moved
        const distance = Math.sqrt(Math.pow(newX - x, 2) + Math.pow(newY - y, 2));
        // Update the position only if the distance moved is greater than the threshold
        if (distance > threshold) {
            setClickPosition({
                x: newX,
                y: newY
            });
            setPopoverOpen(true);
        }
    }
    _react.useEffect(()=>{
        var _barChartSvgRef_current;
        const svgWidth = (barChartSvgRef === null || barChartSvgRef === void 0 ? void 0 : (_barChartSvgRef_current = barChartSvgRef.current) === null || _barChartSvgRef_current === void 0 ? void 0 : _barChartSvgRef_current.getBoundingClientRect().width) || 0;
        const MARGIN_WIDTH_IN_PX = 3;
        if (svgWidth) {
            const currentBarSpacing = MARGIN_WIDTH_IN_PX / svgWidth * 100;
            setBarSpacingInPercent(currentBarSpacing);
        }
    }, [
        barChartSvgRef
    ]);
    const { data } = props;
    _adjustProps();
    const classes = (0, _useHorizontalBarChartStylesstyles.useHorizontalBarChartStyles_unstable)(props);
    const focusAttributes = (0, _reacttabster.useFocusableGroup)();
    let datapoint = 0;
    var _props_culture;
    return !_isChartEmpty() ? /*#__PURE__*/ _react.createElement("div", {
        className: classes.root,
        onMouseLeave: _handleChartMouseLeave
    }, data.map((points, index)=>{
        if (points.chartData && points.chartData[0] && points.chartData[0].horizontalBarChartdata.x) {
            datapoint = points.chartData[0].horizontalBarChartdata.x;
        } else {
            datapoint = 0;
        }
        points.chartData[1] = {
            legend: '',
            horizontalBarChartdata: {
                x: points.chartData[0].horizontalBarChartdata.y - datapoint,
                y: points.chartData[0].horizontalBarChartdata.y
            },
            color: _reacttheme.tokens.colorBackgroundOverlay
        };
        // Hide right side text of chart title for absolute-scale variant
        const chartDataText = props.variant === _index.HorizontalBarChartVariant.AbsoluteScale ? null : _getChartDataText(points);
        const bars = _createBars(points);
        const keyVal = _uniqLineText + '_' + index;
        // ToDo - Showtriangle property is per data series. How to account for it in the new stylesheet
        /*         const classes = useHorizontalBarChartStyles_unstable(props.styles!, {
          width: props.width,
          showTriangle: !!points!.chartData![0].data,
          variant: props.variant,
        }); */ return /*#__PURE__*/ _react.createElement("div", {
            key: index
        }, /*#__PURE__*/ _react.createElement("div", {
            className: classes.items,
            ...focusAttributes
        }, /*#__PURE__*/ _react.createElement("div", {
            className: classes.chartTitle
        }, points.chartTitle && /*#__PURE__*/ _react.createElement(_FocusableTooltipText.FocusableTooltipText, {
            className: classes.chartTitleLeft,
            content: points.chartTitle,
            accessibilityData: points.chartTitleAccessibilityData
        }), chartDataText), points.chartData[0].data && _createBenchmark(points), /*#__PURE__*/ _react.createElement("svg", {
            ref: barChartSvgRef,
            className: classes.chart,
            "aria-label": points.chartTitle
        }, /*#__PURE__*/ _react.createElement("g", {
            id: keyVal,
            key: keyVal,
            ref: (e)=>{
                _refCallback(e, points.chartData[0].legend);
            },
            // NOTE: points.chartData![0] contains current data value
            onClick: ()=>{
                const p = points.chartData[0];
                if (p && p.onClick) {
                    p.onClick();
                }
            }
        }, bars))));
    }), /*#__PURE__*/ _react.createElement(_ChartPopover.ChartPopover, {
        xCalloutValue: xCalloutValue,
        yCalloutValue: yCalloutValue,
        culture: (_props_culture = props.culture) !== null && _props_culture !== void 0 ? _props_culture : 'en-us',
        clickPosition: clickPosition,
        isPopoverOpen: isPopoverOpen,
        legend: legend,
        YValue: hoverValue,
        color: lineColor,
        isCalloutForStack: false,
        customCallout: {
            customizedCallout: props.onRenderCalloutPerHorizontalBar ? props.onRenderCalloutPerHorizontalBar(barCalloutProps) : undefined,
            customCalloutProps: props.customProps ? props.customProps(barCalloutProps) : undefined
        },
        isCartesian: false
    })) : /*#__PURE__*/ _react.createElement("div", {
        id: _emptyChartId,
        role: 'alert',
        style: {
            opacity: '0'
        },
        "aria-label": 'Graph has no data to display'
    });
//TODO validate and fix focus border for issue for popover
});
HorizontalBarChart.displayName = 'HorizontalBarChart';
