"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "Legends", {
    enumerable: true,
    get: function() {
        return Legends;
    }
});
const _interop_require_wildcard = require("@swc/helpers/_/_interop_require_wildcard");
const _react = /*#__PURE__*/ _interop_require_wildcard._(require("react"));
const _reactbutton = require("@fluentui/react-button");
const _shape = require("./shape");
const _useLegendsStylesstyles = require("./useLegendsStyles.styles");
const _reactoverflow = require("@fluentui/react-overflow");
const _reacttabster = require("@fluentui/react-tabster");
const _OverflowMenu = require("./OverflowMenu");
const _reacttheme = require("@fluentui/react-theme");
const Legends = /*#__PURE__*/ _react.forwardRef((props, forwardedRef)=>{
    /** Boolean variable to check if one or more legends are selected */ let _isLegendSelected = false;
    // set states separately for each instance of the component
    const [activeLegend, setActiveLegend] = _react.useState('');
    const [selectedLegends, setSelectedLegends] = _react.useState({});
    const focusAttributes = (0, _reacttabster.useFocusableGroup)();
    const arrowAttributes = (0, _reacttabster.useArrowNavigationGroup)({
        axis: 'horizontal',
        memorizeCurrent: true
    });
    _react.useEffect(()=>{
        let defaultSelectedLegends = {};
        if (props.canSelectMultipleLegends) {
            var _props_defaultSelectedLegends;
            defaultSelectedLegends = ((_props_defaultSelectedLegends = props.defaultSelectedLegends) === null || _props_defaultSelectedLegends === void 0 ? void 0 : _props_defaultSelectedLegends.reduce((combinedDict, key)=>({
                    [key]: true,
                    ...combinedDict
                }), {})) || {};
        } else if (props.defaultSelectedLegend) {
            defaultSelectedLegends = {
                [props.defaultSelectedLegend]: true
            };
        }
        setSelectedLegends(defaultSelectedLegends);
    }, [
        props.canSelectMultipleLegends,
        props.defaultSelectedLegend,
        props.defaultSelectedLegends
    ]);
    _isLegendSelected = Object.keys(selectedLegends).length > 0;
    const dataToRender = _generateData();
    const { overflowStyles, allowFocusOnLegends = true, canSelectMultipleLegends = false } = props;
    const classes = (0, _useLegendsStylesstyles.useLegendStyles_unstable)(props);
    const itemIds = dataToRender.map((_item, index)=>index.toString());
    const overflowHoverCardLegends = [];
    props.legends.map((legend, index)=>{
        const hoverCardElement = _renderButton(legend, index);
        overflowHoverCardLegends.push(hoverCardElement);
    });
    const overflowString = props.overflowText ? props.overflowText : 'more';
    return props.enabledWrapLines ? renderWrappedLegends() : renderLegends();
    function renderLegends() {
        return /*#__PURE__*/ _react.createElement("div", {
            ...focusAttributes,
            ...arrowAttributes,
            ...allowFocusOnLegends && {
                role: 'listbox',
                'aria-label': 'Legends',
                'aria-multiselectable': canSelectMultipleLegends
            },
            className: classes.root
        }, /*#__PURE__*/ _react.createElement(_reactoverflow.Overflow, null, /*#__PURE__*/ _react.createElement("div", {
            className: classes.resizableArea,
            style: {
                textAlign: props.centerLegends ? 'center' : 'unset'
            }
        }, dataToRender.map((item, id)=>/*#__PURE__*/ _react.createElement(_reactoverflow.OverflowItem, {
                key: id,
                id: id.toString()
            }, _renderButton(item))), /*#__PURE__*/ _react.createElement(_OverflowMenu.OverflowMenu, {
            itemIds: itemIds,
            title: `${overflowString}`,
            items: overflowHoverCardLegends
        }))));
    }
    function renderWrappedLegends() {
        return /*#__PURE__*/ _react.createElement("div", {
            ...focusAttributes,
            ...arrowAttributes,
            ...allowFocusOnLegends && {
                role: 'listbox',
                'aria-label': 'Legends',
                'aria-multiselectable': canSelectMultipleLegends
            },
            style: {
                justifyContent: props.centerLegends ? 'center' : 'unset',
                flexWrap: 'wrap',
                ...overflowStyles
            },
            className: classes.root
        }, /*#__PURE__*/ _react.createElement("div", {
            className: classes.resizableArea,
            style: {
                display: 'flex',
                flexWrap: 'wrap',
                overflow: 'auto'
            }
        }, dataToRender.map((item, id)=>/*#__PURE__*/ _react.createElement("div", {
                key: id,
                style: {
                    flex: '0 1 auto',
                    margin: '4px'
                }
            }, _renderButton(item)))));
    }
    function _generateData() {
        const { shape } = props;
        const dataItems = props.legends.map((legend, index)=>{
            return {
                /* ...(allowFocusOnLegends && {
            nativeButtonProps: getIntrinsicElementProps(
              'div',
              {
                legend,
                ...buttonProperties,
              },
              ['title'],
            ),
            'aria-setsize': props.legends.length,
            'aria-posinset': index + 1,
          }), */ title: legend.title,
                action: legend.action,
                hoverAction: legend.hoverAction,
                onMouseOutAction: legend.onMouseOutAction,
                color: legend.color,
                shape: shape ? shape : legend.shape,
                stripePattern: legend.stripePattern,
                isLineLegendInBarChart: legend.isLineLegendInBarChart,
                opacity: legend.opacity,
                key: index
            };
        });
        return dataItems;
    }
    /**
     * This function will get called when there is an ability to
     * select  multiple legends
     * @param legend ILegend
     */ function _canSelectMultipleLegends(legend) {
        let legendsSelected = {
            ...selectedLegends
        };
        if (legendsSelected[legend.title]) {
            // Delete entry for the deselected legend to make
            // the number of keys equal to the number of selected legends
            delete legendsSelected[legend.title];
        } else {
            legendsSelected[legend.title] = true;
            // Clear set if all legends are selected
            if (Object.keys(legendsSelected).length === props.legends.length) {
                legendsSelected = {};
            }
        }
        setSelectedLegends(legendsSelected);
        return legendsSelected;
    }
    /**
     * This function will get called when there is
     * ability to select only single legend
     * @param legend ILegend
     */ function _canSelectOnlySingleLegend(legend) {
        if (selectedLegends[legend.title]) {
            setSelectedLegends({});
            return false;
        } else {
            setSelectedLegends({
                [legend.title]: true
            });
            return true;
        }
    }
    function _onClick(legend, event) {
        var _props_onChange, _legend_action;
        const { canSelectMultipleLegends = false } = props;
        let selectedLegends = [];
        if (canSelectMultipleLegends) {
            const nextSelectedLegends = _canSelectMultipleLegends(legend);
            selectedLegends = Object.keys(nextSelectedLegends);
        } else {
            const isSelected = _canSelectOnlySingleLegend(legend);
            selectedLegends = isSelected ? [
                legend.title
            ] : [];
        }
        (_props_onChange = props.onChange) === null || _props_onChange === void 0 ? void 0 : _props_onChange.call(props, selectedLegends, event, legend);
        (_legend_action = legend.action) === null || _legend_action === void 0 ? void 0 : _legend_action.call(legend);
    }
    function _onHoverOverLegend(legend) {
        if (legend.hoverAction) {
            setActiveLegend(legend.title);
            legend.hoverAction();
        }
    }
    function _onLeave(legend) {
        if (legend.onMouseOutAction) {
            setActiveLegend('');
            legend.onMouseOutAction();
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    function _renderButton(data, index) {
        const { allowFocusOnLegends = true } = props;
        const legend = {
            title: data.title,
            color: data.color,
            shape: data.shape,
            action: data.action,
            hoverAction: data.hoverAction,
            onMouseOutAction: data.onMouseOutAction,
            stripePattern: data.stripePattern,
            isLineLegendInBarChart: data.isLineLegendInBarChart,
            opacity: data.opacity
        };
        const color = _getColor(legend.title, legend.color);
        const onClickHandler = (event)=>{
            _onClick(legend, event);
        };
        const onHoverHandler = ()=>{
            _onHoverOverLegend(legend);
        };
        const onMouseOut = ()=>{
            _onLeave(legend);
        };
        const shape = _getShape(legend, color);
        return /*#__PURE__*/ _react.createElement(_reactbutton.Button, {
            ...allowFocusOnLegends && {
                'aria-selected': !!selectedLegends[legend.title],
                role: 'option',
                'aria-label': `${legend.title}`,
                'aria-setsize': data['aria-setsize'],
                'aria-posinset': data['aria-posinset']
            },
            ...data.nativeButtonProps && {
                ...data.nativeButtonProps
            },
            key: index,
            className: classes.legend,
            onClick: onClickHandler,
            onMouseOver: onHoverHandler,
            onMouseOut: onMouseOut,
            onFocus: onHoverHandler,
            onBlur: onMouseOut,
            appearance: 'outline',
            size: "small",
            style: {
                '--rect-height': legend.isLineLegendInBarChart ? '4px' : '12px',
                '--rect-backgroundColor': legend.stripePattern ? '' : color,
                '--rect-borderColor': legend.color ? legend.color : _reacttheme.tokens.colorNeutralStroke1,
                '--rect-content': legend.stripePattern ? `repeating-linear-gradient(135deg, transparent, transparent 3px, ${color} 1px, ${color} 4px)` : ''
            }
        }, shape, /*#__PURE__*/ _react.createElement("div", {
            className: classes.text,
            style: {
                opacity: color === _reacttheme.tokens.colorNeutralBackground1 ? '0.67' : ''
            }
        }, legend.title));
    }
    function _getShape(legend, color) {
        const svgParentProps = {
            className: classes.shape
        };
        const svgChildProps = {
            fill: color,
            strokeWidth: 2,
            stroke: legend.color
        };
        return /*#__PURE__*/ _react.createElement(_shape.Shape, {
            svgProps: svgParentProps,
            pathProps: svgChildProps,
            shape: legend.shape,
            classNameForNonSvg: classes.rect,
            style: {
                height: legend.isLineLegendInBarChart ? '4px' : '12px',
                backgroundColor: legend.stripePattern ? '' : color,
                borderColor: legend.color ? legend.color : _reacttheme.tokens.colorNeutralStroke1,
                content: legend.stripePattern ? `repeating-linear-gradient(135deg, transparent, transparent 3px, ${color} 1px, ${color} 4px)` : '',
                '--rect-content-high-contrast': `linear-gradient(to right, ${color}, ${color})`,
                '--rect-opacity-high-contrast': color === _reacttheme.tokens.colorNeutralBackground1 ? '0.6' : ''
            }
        });
    }
    function _getColor(title, color) {
        let legendColor = color;
        // if one or more legends are selected
        if (_isLegendSelected) {
            // if the given legend (title) is one of the selected legends
            if (selectedLegends[title]) {
                legendColor = color;
            } else {
                legendColor = _reacttheme.tokens.colorNeutralBackground1;
            }
        } else {
            // if the given legend is hovered
            // or none of the legends is hovered
            if (activeLegend === title || activeLegend === '') {
                legendColor = color;
            } else {
                legendColor = _reacttheme.tokens.colorNeutralBackground1;
            }
        }
        return legendColor;
    }
});
Legends.displayName = 'Legends';
