"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "SVGTooltipText", {
    enumerable: true,
    get: function() {
        return SVGTooltipText;
    }
});
const _interop_require_wildcard = require("@swc/helpers/_/_interop_require_wildcard");
const _react = /*#__PURE__*/ _interop_require_wildcard._(require("react"));
const _reactsharedcontexts = require("@fluentui/react-shared-contexts");
const _reacttooltip = require("@fluentui/react-tooltip");
const _asyncutils = require("./async-utils");
const _KeyCodes = require("./KeyCodes");
const _reactutilities = require("@fluentui/react-utilities");
const SVGTooltipText = /*#__PURE__*/ _react.forwardRef((props, forwardedRef)=>{
    const [isTooltipVisible, setIsTooltipVisible] = (0, _react.useState)(false);
    const [isOverflowing, setIsOverflowing] = (0, _react.useState)(false);
    const tooltipHostRef = (0, _react.useRef)(null);
    const async = (0, _react.useRef)(new _asyncutils.Async()).current;
    const dismissTimerId = (0, _react.useRef)();
    const openTimerId = (0, _react.useRef)();
    const tooltipHostId = (0, _react.useRef)((0, _reactutilities.useId)('tooltip-host')).current;
    const ignoreNextFocusEvent = (0, _react.useRef)(false);
    const portalMountNode = (0, _reactsharedcontexts.usePortalMountNode)();
    const wrapContentCallback = (0, _react.useCallback)(()=>{
        var _props_maxWidth;
        if (props.content && props.wrapContent && props.wrapContent(props.content, tooltipHostId, (_props_maxWidth = props.maxWidth) !== null && _props_maxWidth !== void 0 ? _props_maxWidth : 100, props.maxHeight) // ToDo - Specify a correct fallback value here
        ) {
            setIsOverflowing(true);
        } else {
            setIsOverflowing(false);
        }
    }, [
        props,
        tooltipHostId
    ]);
    (0, _react.useEffect)(()=>{
        wrapContentCallback();
        return ()=>{
            async.dispose();
        };
    }, [
        wrapContentCallback,
        async
    ]);
    (0, _react.useEffect)(()=>{
        wrapContentCallback();
    }, [
        props.maxWidth,
        props.maxHeight,
        wrapContentCallback
    ]);
    const hideTooltip = (0, _react.useCallback)(()=>{
        async.clearTimeout(openTimerId.current);
        async.clearTimeout(dismissTimerId.current);
        setIsTooltipVisible(false);
    }, [
        async
    ]);
    const onTooltipMouseEnter = (0, _react.useCallback)((ev)=>{
        if (!isOverflowing) {
            return;
        }
        if (ev.target && (portalMountNode === null || portalMountNode === void 0 ? void 0 : portalMountNode.contains(ev.target))) {
            return;
        }
        async.clearTimeout(dismissTimerId.current);
        async.clearTimeout(openTimerId.current);
        if (props.delay !== 0) {
            openTimerId.current = async.setTimeout(()=>{
                setIsTooltipVisible(true);
            }, props.delay);
        } else {
            setIsTooltipVisible(true);
        }
    }, [
        isOverflowing,
        portalMountNode,
        async,
        props.delay
    ]);
    const onTooltipMouseLeave = (0, _react.useCallback)((ev)=>{
        async.clearTimeout(dismissTimerId.current);
        async.clearTimeout(openTimerId.current);
        if (props.closeDelay) {
            dismissTimerId.current = async.setTimeout(()=>{
                setIsTooltipVisible(false);
            }, props.closeDelay);
        } else {
            setIsTooltipVisible(false);
        }
    }, [
        async,
        props.closeDelay
    ]);
    const onTooltipFocus = (0, _react.useCallback)((ev)=>{
        if (ignoreNextFocusEvent.current) {
            ignoreNextFocusEvent.current = false;
            return;
        }
        onTooltipMouseEnter(ev);
    }, [
        onTooltipMouseEnter
    ]);
    const onTooltipBlur = (0, _react.useCallback)((ev)=>{
        var _document;
        ignoreNextFocusEvent.current = ((_document = document) === null || _document === void 0 ? void 0 : _document.activeElement) === ev.target;
        dismissTimerId.current = async.setTimeout(()=>{
            setIsTooltipVisible(false);
        }, 0);
    }, [
        async
    ]);
    const onTooltipKeyDown = (0, _react.useCallback)((ev)=>{
        if ((ev.which === _KeyCodes.KeyCodes.escape || ev.ctrlKey) && isTooltipVisible) {
            hideTooltip();
            ev.stopPropagation();
        }
    }, [
        isTooltipVisible,
        hideTooltip
    ]);
    const showTooltip = props.isTooltipVisibleProp && isOverflowing && !!props.content || isTooltipVisible && !!props.content;
    return /*#__PURE__*/ _react.createElement(_react.Fragment, null, /*#__PURE__*/ _react.createElement(_reacttooltip.Tooltip, {
        relationship: "description",
        ...props.tooltipProps,
        withArrow: true,
        content: props.content,
        // targetElement={getTargetElement()} ToDo - This assignment is causing build failure. Needs to be fixed.
        visible: showTooltip
    }, /*#__PURE__*/ _react.createElement("text", {
        ...props.textProps,
        id: tooltipHostId,
        ref: tooltipHostRef,
        onFocusCapture: onTooltipFocus,
        onBlurCapture: onTooltipBlur,
        onMouseEnter: onTooltipMouseEnter,
        onMouseLeave: onTooltipMouseLeave,
        onKeyDown: onTooltipKeyDown,
        "data-is-focusable": props.shouldReceiveFocus && isOverflowing
    }, props.content)));
});
SVGTooltipText.defaultProps = {
    delay: 0
};
