import * as React from 'react';
import { useFluent_unstable } from '@fluentui/react-shared-contexts';
import { useResponsiveContainerStyles_unstable } from './useResponsiveContainerStyles.styles';
export const ResponsiveContainer = (props)=>{
    const containerRef = React.useRef(null);
    const onResizeRef = React.useRef();
    const { targetDocument } = useFluent_unstable();
    const classes = useResponsiveContainerStyles_unstable(props);
    const [size, setSize] = React.useState({});
    onResizeRef.current = props.onResize;
    const _window = targetDocument === null || targetDocument === void 0 ? void 0 : targetDocument.defaultView;
    React.useEffect(()=>{
        let animationFrameId;
        // eslint-disable-next-line no-restricted-globals
        let resizeObserver;
        const resizeCallback = (entries)=>{
            var _onResizeRef_current;
            const { width: containerWidth, height: containerHeight } = entries[0].contentRect;
            // rAF is an alternative to the throttle function. For more info, see:
            // https://css-tricks.com/debouncing-throttling-explained-examples/#aa-requestanimationframe-raf
            animationFrameId = _window === null || _window === void 0 ? void 0 : _window.requestAnimationFrame(()=>{
                setSize((prevSize)=>{
                    const roundedWidth = Math.floor(containerWidth);
                    const roundedHeight = Math.floor(containerHeight);
                    if (prevSize.containerWidth === roundedWidth && prevSize.containerHeight === roundedHeight) {
                        return prevSize;
                    }
                    return {
                        containerWidth: roundedWidth,
                        containerHeight: roundedHeight
                    };
                });
            });
            (_onResizeRef_current = onResizeRef.current) === null || _onResizeRef_current === void 0 ? void 0 : _onResizeRef_current.call(onResizeRef, containerWidth, containerHeight);
        };
        if (_window === null || _window === void 0 ? void 0 : _window.ResizeObserver) {
            resizeObserver = new _window.ResizeObserver(resizeCallback);
            if (containerRef.current) {
                resizeObserver.observe(containerRef.current);
            }
        }
        return ()=>{
            if (animationFrameId) {
                _window === null || _window === void 0 ? void 0 : _window.cancelAnimationFrame(animationFrameId);
            }
            resizeObserver === null || resizeObserver === void 0 ? void 0 : resizeObserver.disconnect();
        };
    }, [
        _window
    ]);
    return /*#__PURE__*/ React.createElement("div", {
        ref: containerRef,
        className: classes.root,
        style: {
            width: props.width,
            height: props.height
        }
    }, props.children(size));
};
ResponsiveContainer.displayName = 'ResponsiveContainer';
